package geniusweb.protocol.session;

import java.util.List;

import com.fasterxml.jackson.annotation.JsonSubTypes;

import geniusweb.protocol.NegoSettings;
import geniusweb.protocol.session.amop.AMOPSettings;
import geniusweb.protocol.session.mopac.MOPACSettings;
import geniusweb.protocol.session.saop.SAOPSettings;
import geniusweb.protocol.session.shaop.SHAOPSettings;
import geniusweb.references.PartyWithProfile;
import tudelft.utilities.logging.Reporter;

/**
 * interface for settings for session protocols. Immutable.
 */
@JsonSubTypes({ @JsonSubTypes.Type(value = SAOPSettings.class),
		@JsonSubTypes.Type(value = SHAOPSettings.class),
		@JsonSubTypes.Type(value = AMOPSettings.class),
		@JsonSubTypes.Type(value = MOPACSettings.class) })
public interface SessionSettings extends NegoSettings {

	/**
	 * Get participants
	 * 
	 * @return list of {@link PartyWithProfile} items.
	 */
	List<TeamOfPartiesAndProfiles> getTeams();

	/**
	 * Allows modification of SessionSettings to include a party. This is needed
	 * for tournament auto-configuration of sessions.
	 * 
	 * @param partyprofteam the {@link TeamOfPartiesAndProfiles} to be added
	 * @return new modified SessionSettings object
	 */
	SessionSettings with(TeamOfPartiesAndProfiles partyprofteam);

	/**
	 * 
	 * @return all parties from all teams, as a flattened ordered list. The
	 *         order : start with all parties from team 1, then all from team 2,
	 *         etc. The specific order of the parties from a team depends on the
	 *         protocol.
	 */
	List<PartyWithProfile> getAllParties();

	/**
	 * @param logger the logger where the protocol can log events to.
	 * @return the an initialized and ready to use {@link SessionProtocol} that
	 *         can handle this Negotiation.
	 */
	@Override
	SessionProtocol getProtocol(Reporter logger);

}
